@@ -22,12 +22,15 @@
 enum ofono_vendor {
 	OFONO_VENDOR_GENERIC = 0,
 	OFONO_VENDOR_CALYPSO,
+	OFONO_VENDOR_IFX,
 	OFONO_VENDOR_STE,
 	OFONO_VENDOR_MBM,
 	OFONO_VENDOR_QUALCOMM_MSM,
 	OFONO_VENDOR_OPTION_HSO,
+	OFONO_VENDOR_ZTE,
 	OFONO_VENDOR_HUAWEI,
 	OFONO_VENDOR_NOVATEL,
 	OFONO_VENDOR_WAVECOM,
+	OFONO_VENDOR_NOKIA,
 	OFONO_VENDOR_PHONESIM,
 };
